package com.example.attestr_flowx_plugin;

import android.app.Activity;
import android.text.TextUtils;
import android.util.Log;

import androidx.annotation.NonNull;

import com.attestr.flowx.AttestrFlowx;
import com.attestr.flowx.listener.AttestrFlowXListener;

import java.util.HashMap;
import java.util.Map;

import io.flutter.embedding.android.FlutterActivity;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/** AttestrFlowxPlugin */
public class AttestrFlowxPlugin extends FlutterActivity
        implements FlutterPlugin, MethodCallHandler, ActivityAware, AttestrFlowXListener {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity

  private static final String CHANNEL_NAME = "attestr_flowx_plugin";
  private MethodChannel channel;
//  private AttestrFlowxDelegate attestrFlowxDelegate;

  private static final int FLOW_COMPLETE = 0;
  private static final int FLOW_SKIP = 1;
  private static final int FLOW_ERROR = 2;
  private static final String TAG = "AttestrFlowxDelegate";
  private final AttestrFlowx attestrFlowx = new AttestrFlowx();
  private Result pendingResult;
  private FlutterActivity mActivity;

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), CHANNEL_NAME);
    channel.setMethodCallHandler(this);
  }

  @Override
  public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
//    channel.setMethodCallHandler((call, result) -> {
//      if (call.method.equals("getPlatformVersion")) {
//        result.success("Android " + android.os.Build.VERSION.RELEASE);
//      } else if (call.method.equals("initiateSession")) {
//        AttestrFlowxDelegate attestrFlowxDelegate = new AttestrFlowxDelegate(binding.getActivity());
//        attestrFlowxDelegate.initiateSession(call, result);
//      } else {
//        result.notImplemented();
//      }
//    });
//    attestrFlowxDelegate = new AttestrFlowxDelegate(binding.getActivity());
    this.mActivity = (FlutterActivity) binding.getActivity();
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    if (call.method.equals("getPlatformVersion")) {
      result.success("Android " + android.os.Build.VERSION.RELEASE);
    } else if (call.method.equals("initiateSession")) {
      initiateSession(call, result);
    } else {
      result.notImplemented();
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {
    onDetachedFromActivity();
  }

  @Override
  public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
    onAttachedToActivity(binding);
  }

  @Override
  public void onDetachedFromActivity() {

  }

  public void initiateSession(@NonNull MethodCall call,
                              @NonNull Result result) {
    this.pendingResult = result;
    String handshakeID = call.argument("handshake_id");
    String client_key = call.argument("client_key");
    String locale = call.argument("locale");
    boolean retry = Boolean.TRUE.equals(call.argument("retry"));
    Map<String, String> queryParams = call.argument("qr");

    if (mActivity != null) {
      if (!TextUtils.isEmpty(handshakeID) && !TextUtils.isEmpty(client_key)) {
        attestrFlowx.init(client_key, handshakeID, mActivity);
        attestrFlowx.launch(
                locale,
                retry,
                queryParams
        );
      }
    } else {
      Log.d(TAG, "Activity Null");
    }
  }

  @Override
  public void onFlowXComplete(Map<String, Object> map) {
    if (pendingResult != null) {
      Map<String, Object> reply = new HashMap<>();
      reply.put("type", FLOW_COMPLETE);

      Map<String, Object> data = new HashMap<>();
      data.put("message", map.get("signature"));

      reply.put("data", data);

      pendingResult.success(reply);
    }
  }

  @Override
  public void onFlowXSkip(Map<String, Object> map) {
    if (pendingResult != null) {
      Map<String, Object> reply = new HashMap<>();
      reply.put("type", FLOW_SKIP);

      Map<String, Object> data = new HashMap<>();
      data.put("message", "Flow skip");

      reply.put("data", data);

      pendingResult.success(reply);
    }
  }

  @Override
  public void onFlowXError(Map<String, Object> map) {
    if (pendingResult != null) {
      Map<String, Object> reply = new HashMap<>();
      reply.put("type", FLOW_ERROR);

      Map<String, Object> data = new HashMap<>();
      data.put("message", map.get("message"));

      reply.put("data", data);

      pendingResult.success(reply);
    }
  }
}
